@@ -187,6 +187,7 @@ TYPEREAL f = InRate;
 	{
 		m_InRate = InRate;
 		m_MaxBW = MaxBW;
+		m_Mutex.lock();
 		DeleteFilters();
 		//loop until closest output rate is found and list of pointers to decimate by 2 stages is generated
 		while( (f > 400000.0)  )
@@ -195,6 +196,7 @@ TYPEREAL f = InRate;
 			f /= 2.0;
 		}
 		m_OutputRate = f;
+		m_Mutex.unlock();
 		SetFrequency(m_NcoFreq);
 	}
 	return m_OutputRate;
@@ -281,7 +283,7 @@ TYPEREAL*	pdSinAns  = &dASMSin;
 	{
 		n = m_pDecimatorPtrs[j++]->DecBy2(n, pInData, pInData);
 //if(1==j)
-//g_pTestBench->DisplayData(n, (TYPECPX*)pInData, 615385/2.0);
+//g_pTestBench->DisplayData(n, 1.0, (TYPECPX*)pInData, 615385/2.0);
 	}
 	m_Mutex.unlock();
 	for(i=0; i<n; i++)
