@@ -7,6 +7,8 @@
 //	2011-09-17  Initial creation MSW
 //	2011-09-17  Initial release
 //	2013-07-28  Added single/double precision math macros
+//	2014-09-22  Added some test code to output to a wav file
+//	2016-01-10  removed x86 assembly code
 //////////////////////////////////////////////////////////////////////
 
 //==========================================================================================
@@ -95,6 +97,71 @@ const TYPEREAL HILBLP_H[HILB_LENGTH] =
 	-0.000389631665953405
 };
 
+#if 0
+const TYPEREAL HILBLP_H[HILB_LENGTH] = {	//test wideband hilbert
+	0.000639403635f,
+   -0.000876414761f,
+	0.000763344477f,
+   -0.000136153196f,
+   -0.000959189633f,
+	0.002196797759f,
+   -0.003022642082f,
+	0.002843677701f,
+   -0.001305921508f,
+   -0.001445249432f,
+	0.004652418499f,
+   -0.007079194912f,
+	0.007402035698f,
+   -0.004761371611f,
+   -0.000723462240f,
+	0.007705168551f,
+   -0.013799667280f,
+	0.016252251992f,
+   -0.012930808533f,
+	0.003338459172f,
+	0.010731465462f,
+   -0.025212726547f,
+	0.034579859350f,
+   -0.033317627946f,
+	0.017655163815f,
+	0.012969531446f,
+   -0.055222198103f,
+	0.102266070453f,
+   -0.145217247362f,
+	0.175301692976f,
+	0.813425068326f,
+	0.175301692976f,
+   -0.145217247362f,
+	0.102266070453f,
+   -0.055222198103f,
+	0.012969531446f,
+	0.017655163815f,
+   -0.033317627946f,
+	0.034579859350f,
+   -0.025212726547f,
+	0.010731465462f,
+	0.003338459172f,
+   -0.012930808533f,
+	0.016252251992f,
+   -0.013799667280f,
+	0.007705168551f,
+   -0.000723462240f,
+   -0.004761371611f,
+	0.007402035698f,
+   -0.007079194912f,
+	0.004652418499f,
+   -0.001445249432f,
+   -0.001305921508f,
+	0.002843677701f,
+   -0.003022642082f,
+	0.002196797759f,
+   -0.000959189633f,
+   -0.000136153196f,
+	0.000763344477f,
+   -0.000876414761f,
+	0.000639403635f
+};
+#endif
 
 /////////////////////////////////////////////////////////////////////////////////
 //	Construct/destruct WFM demod object
@@ -114,6 +181,13 @@ CWFmDemod::CWFmDemod(TYPEREAL samplerate) : m_SampleRate(samplerate)
     m_PilotLocked = false;
 	m_LastPilotLocked = !m_PilotLocked;
 	m_BlockErrors = 0;
+
+	QAudioFormat format;
+	format.setChannelCount(2);
+	format.setSampleSize(16);
+	format.setSampleRate(245760);
+	format.setCodec("audio/pcm");
+	format.setSampleType(QAudioFormat::SignedInt);
 }
 
 CWFmDemod::~CWFmDemod()
@@ -214,7 +288,7 @@ qDebug()<<"RDS Rate = "<< m_RdsOutputRate;
 int CWFmDemod::ProcessData(int InLength, TYPECPX* pInData, TYPEREAL* pOutData)
 {
 	m_MonoLPFilter.ProcessFilter(InLength,pInData, pInData);
-g_pTestBench->DisplayData(InLength, pInData, m_SampleRate,PROFILE_2);
+g_pTestBench->DisplayData(InLength, 1.0, pInData, m_SampleRate,PROFILE_2);
 	for(int i=0; i<InLength; i++)
 	{
 		m_D0 = pInData[i];
@@ -229,7 +303,7 @@ g_pTestBench->DisplayData(InLength, pInData, m_SampleRate,PROFILE_2);
 	if(m_pDecBy2C)
 		InLength = m_pDecBy2C->DecBy2(InLength, pOutData, pOutData);
 
-//g_pTestBench->DisplayData(InLength, m_RawFm, m_SampleRate,PROFILE_2);
+//g_pTestBench->DisplayData(InLength, 1.0, m_RawFm, m_SampleRate,PROFILE_2);
 
 	m_LPFilter.ProcessFilter( InLength, pOutData, pOutData);	//rolloff audio above 15KHz
 	ProcessDeemphasisFilter(InLength, pOutData, pOutData);		//50 or 75uSec de-emphasis one pole filter
@@ -245,7 +319,7 @@ g_pTestBench->DisplayData(InLength, pInData, m_SampleRate,PROFILE_2);
 // Perform wideband FM demod into a REAL data stream.
 // Perform REAL to complex filtering to make easier to shift and process signals
 //		within the demodulated FM signal.
-// IIR Filter sround the 19KHz Pilot then Phase Lock a PLL to it.
+// IIR Filter around the 19KHz Pilot then Phase Lock a PLL to it.
 // If locked, perform stereo demuxing using the PLL signal and a delay line to
 //		match the raw REAL data stream.
 // Shift the 57KHz RDS signal to baseband and decimate its sample rate down.
@@ -266,15 +340,19 @@ TYPEREAL LminusR;
 	for(int i=0; i<InLength; i++)
 	{
 		m_D0 = pInData[i];
-		m_RawFm[i] = FMDEMOD_GAIN*MATAN2( (m_D1.re*m_D0.im - m_D0.re*m_D1.im), (m_D1.re*m_D0.re + m_D1.im*m_D0.im));//~266 nSec/sample
+//        m_RawFm[i] = FMDEMOD_GAIN*MATAN2( (m_D1.re*m_D0.im - m_D0.re*m_D1.im), (m_D1.re*m_D0.re + m_D1.im*m_D0.im));//~266 nSec/sample
+        m_RawFm[i] = FMDEMOD_GAIN*arctan2( (m_D1.re*m_D0.im - m_D0.re*m_D1.im), (m_D1.re*m_D0.re + m_D1.im*m_D0.im));//~266 nSec/sample
+
 		m_D1 = m_D0;
 	}
 //StopPerformance(InLength);
 
-g_pTestBench->DisplayData(InLength, m_RawFm, m_SampleRate,PROFILE_2);
+g_pTestBench->DisplayData(InLength, 1.0, m_RawFm, m_SampleRate,PROFILE_2);
 	//create complex data from demodulator real data
 	m_HilbertFilter.ProcessFilter(InLength, m_RawFm, m_CpxRawFm);	//~173 nSec/sample
-//g_pTestBench->DisplayData(InLength, m_CpxRawFm, m_SampleRate,PROFILE_3);
+
+
+//g_pTestBench->DisplayData(InLength, 1.0, m_CpxRawFm, m_SampleRate,PROFILE_3);
 
 	m_PilotBPFilter.ProcessFilter(InLength, m_CpxRawFm, pInData);//~173 nSec/sample, use input buffer for complex output storage
 	if(ProcessPilotPll(InLength, pInData) )
@@ -303,25 +381,25 @@ g_pTestBench->DisplayData(InLength, m_RawFm, m_SampleRate,PROFILE_2);
 	int length = m_RdsDownConvert.ProcessData(InLength, m_CpxRawFm, m_RdsRaw);
 
 	//filter baseband RDS signal
-//g_pTestBench->DisplayData(length, m_RdsRaw, m_RdsOutputRate, PROFILE_3);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsRaw, m_RdsOutputRate, PROFILE_3);
 	m_RdsBPFilter.ProcessFilter(length, m_RdsRaw, m_RdsRaw);
 //g_pTestBench->DisplayData(length, m_RdsRaw, m_RdsOutputRate, PROFILE_3);
 
 	//PLL to remove any rotation since may not be phase locked to 19KHz Pilot or may not even have pilot
 	ProcessRdsPll(length, m_RdsRaw, m_RdsMag);
-//g_pTestBench->DisplayData(length, m_RdsMag, m_RdsOutputRate, PROFILE_3);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsMag, m_RdsOutputRate, PROFILE_3);
 
 	//run matched filter correlator to extract the bi-phase data bits
 	m_RdsMatchedFilter.ProcessFilter(length,m_RdsMag,m_RdsData);
-//g_pTestBench->DisplayData(length, m_RdsData, m_RdsOutputRate, PROFILE_6);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsData, m_RdsOutputRate, PROFILE_6);
 	//create bit sync signal in m_RdsMag[] by squaring data
 	for(int i=0; i<length; i++)
 		m_RdsMag[i] = m_RdsData[i]* m_RdsData[i];	//has high energy at the bit clock rate and 2x bit rate
 
-//g_pTestBench->DisplayData(length, m_RdsMag, m_RdsOutputRate, PROFILE_6);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsMag, m_RdsOutputRate, PROFILE_6);
 	//run Hi-Q resonator filter that create a sin wave that will lock to BitRate clock and not 2X rate
 	m_RdsBitSyncFilter.ProcessFilter(length, m_RdsMag, m_RdsMag);
-//g_pTestBench->DisplayData(length, m_RdsMag, m_RdsOutputRate, PROFILE_6);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsMag, m_RdsOutputRate, PROFILE_6);
 	//now loop through samples to determine where bit position is and extract binary digital data
 	for(int i=0; i<length; i++)
 	{
@@ -337,12 +415,12 @@ g_pTestBench->DisplayData(InLength, m_RawFm, m_SampleRate,PROFILE_2);
 			if(m_RdsLastData>=0)
 			{
 				bit = 1;
-m_RdsRaw[i].re = m_RdsLastData;
+				m_RdsRaw[i].re = m_RdsLastData;
 			}
 			else
 			{
 				bit = 0;
-m_RdsRaw[i].re = m_RdsLastData;
+				m_RdsRaw[i].re = m_RdsLastData;
 			}
 			//need to XOR with previous bit to get actual data bit value
 			ProcessNewRdsBit(bit^m_RdsLastBit);		//go process new RDS Bit
@@ -350,16 +428,16 @@ m_RdsRaw[i].re = m_RdsLastData;
 		}
 		else
 		{
-m_RdsRaw[i].re = 0;
+			m_RdsRaw[i].re = 0;
 		}
 
 		m_RdsLastData = Data;		//keep last bit since is differential data
 		m_RdsLastSyncSlope = Slope;
-m_RdsRaw[i].im = Data;
+		m_RdsRaw[i].im = Data;
 	}
-g_pTestBench->DisplayData(length, m_RdsRaw, m_RdsOutputRate, PROFILE_3);	//display rds data and sample point
+//g_pTestBench->DisplayData(length, 1.0, m_RdsRaw, m_RdsOutputRate, PROFILE_3);	//display rds data and sample point
 
-//g_pTestBench->DisplayData(length, m_RdsData, m_RdsOutputRate, PROFILE_3);
+//g_pTestBench->DisplayData(length, 1.0, m_RdsData, m_RdsOutputRate, PROFILE_3);
 
 	//decimate by 2's down close to final audio rate
 	if(m_pDecBy2A)
@@ -407,12 +485,8 @@ TYPECPX tmp;
 //m_PilotPhaseAdjust = g_TestValue;
 	for(int i=0; i<InLength; i++)	//175 nSec
 	{
-#if 1
-		asm volatile ("fsincos" : "=%&t" (Cos), "=%&u" (Sin) : "0" (m_PilotNcoPhase));	//126nS
-#else
 		Sin = MSIN(m_PilotNcoPhase);		//178ns for sin/cos calc
 		Cos = MCOS(m_PilotNcoPhase);
-#endif
 		//complex multiply input sample by NCO's  sin and cos
 		tmp.re = Cos * pInData[i].re - Sin * pInData[i].im;
 		tmp.im = Cos * pInData[i].im + Sin * pInData[i].re;
@@ -557,12 +631,8 @@ TYPEREAL Cos;
 TYPECPX tmp;
 	for(int i=0; i<InLength; i++)
 	{
-#if 1
-		asm volatile ("fsincos" : "=%&t" (Cos), "=%&u" (Sin) : "0" (m_RdsNcoPhase));	//126nS
-#else
 		Sin = MSIN(m_RdsNcoPhase);		//178ns for sin/cos calc
 		Cos = MCOS(m_RdsNcoPhase);
-#endif
 		//complex multiply input sample by NCO's  sin and cos
 		tmp.re = Cos * pInData[i].re - Sin * pInData[i].im;
 		tmp.im = Cos * pInData[i].im + Sin * pInData[i].re;
@@ -580,7 +650,7 @@ TYPECPX tmp;
 		pOutData[i] = tmp.im;
 	}
 	m_RdsNcoPhase = MFMOD(m_RdsNcoPhase, K_2PI);	//keep radian counter bounded
-//g_pTestBench->DisplayData(InLength, m_RdsData, m_RdsOutputRate, PROFILE_6);
+//g_pTestBench->DisplayData(InLength, 1.0, m_RdsData, m_RdsOutputRate, PROFILE_6);
 }
 
 
