@@ -22,8 +22,12 @@ public:
 	void InitDlg();
 	bool eventFilter(QObject* o, QEvent* e);
 
+	int m_ActiveHostAdrIndex;
 	QHostAddress m_IPAdr;
 	quint32 m_Port;
+	bool m_UseUdpFwd;
+	QHostAddress m_IPFwdAdr;
+	quint32 m_FwdPort;
 	QString m_ActiveDevice;
 	bool m_DirtyFlag;
 
