@@ -18,6 +18,20 @@
 //	2013-07-28  ver 1.11 Updated to QT 5.10 fixed DisconnectFromServerSlot bug, Added single/double precision math macros
 //	2013-12-16  ver 1.12 Updated to QT 5.20 updated to Q_OS_WIN macro use
 //	2014-02-23  ver 1.13 Updated to correct qwindows.dll for QT 5.2 and expanded frequency ranges
+//	2014-07-11  ver 1.14 Updated for QT 5.3
+//	2014-09-22  ver 1.15 Modified Thread Launcher to deal with closing resources from thread context
+//	2015-02-05  ver 1.15a Added CloudSDR discover fields. no need for release now
+//	2015-02-25  ver 1.16 Added CloudSDR-IQ and added PSK digital decoder
+//	2015-03-26  ver 1.17 Added  support for small MTU and UDP keepalive in case of port forwarding timeouts
+//	2015-06-09  ver 1.18 Fixed discovery issue with CloudIQ
+//	2015-08-27  ver 1.19 Changed Cloudxx max BW to 1.5MHz, added wav file saving
+//	2016-01-10  ver 1.20b1 Removed x86 assembly code in wfmdemod, added settings saving in OnExit() (thanks alex for fix)
+//	2016-01-10  ver 1.20b2 Removed another x86 assembly code in wfmdemod changed StayOnTop inhibit for Linux
+//	2016-10-14  ver 1.20b3 Release candidate
+//	2017-09-09  ver 1.20b4 Added network Interface selection
+//	2017-09-14  ver 1.20b5 Added FSK demod but not finished
+//	2017-12-25  ver 1.20 Cleaned up DSC mode, recompile with Qt 5.10
+
 /////////////////////////////////////////////////////////////////////
 //==========================================================================================
 // + + +   This Software is released under the "Simplified BSD License"  + + +
@@ -59,12 +73,13 @@
 #include "gui/sounddlg.h"
 #include "gui/displaydlg.h"
 #include "gui/aboutdlg.h"
+#include "gui/recordsetupdlg.h"
 #include "interface/perform.h"
 
 /*---------------------------------------------------------------------------*/
 /*--------------------> L O C A L   D E F I N E S <--------------------------*/
 /*---------------------------------------------------------------------------*/
-#define PROGRAM_TITLE_VERSION tr(" 1.13")
+#define PROGRAM_TITLE_VERSION tr(" 1.20")
 
 #define MAX_FFTDB 60
 #define MIN_FFTDB -170
@@ -91,10 +106,19 @@ MainWindow::MainWindow(QWidget *parent) :
 	if(!g_pTestBench)
 		g_pTestBench = new CTestBench(this);
 
-	readSettings();		//read persistent settings
+	if(!g_pChatDialog)
+	{
+		g_pChatDialog = new CChatDialog(this, Qt::WindowTitleHint );
+		g_pChatDialog->SetSdrInterface(m_pSdrInterface);
+	}
 
+	InitDemodSettings();	//must be before readSettings to set some defualts
+	readSettings();			//read persistent settings
+
+#ifndef Q_OS_LINUX
 	ui->actionAlwaysOnTop->setChecked(m_AlwaysOnTop);
 	AlwaysOnTop();
+#endif
 
 	//create Demod setup menu for non-modal use(can leave up and still access rest of program)
 	m_pDemodSetupDlg = new CDemodSetupDlg(this);
@@ -119,6 +143,7 @@ MainWindow::MainWindow(QWidget *parent) :
 	connect(ui->actionAlwaysOnTop, SIGNAL(triggered()), this, SLOT(AlwaysOnTop()));
 	connect(ui->actionDemod_Setup, SIGNAL(triggered()), this, SLOT(OnDemodDlg()));
 	connect(ui->actionNoise_Processing, SIGNAL(triggered()), this, SLOT(OnNoiseProcDlg()));
+	connect(ui->actionRecordSetup,SIGNAL(triggered()), this, SLOT(OnRecordSetupDlg()));
 
 	connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(OnAbout()));
 
@@ -168,7 +193,6 @@ MainWindow::MainWindow(QWidget *parent) :
 
 	ui->framePlot->SetSpanFreq( m_SpanFrequency );
 	ui->framePlot->SetCenterFreq( m_CenterFrequency );
-	ui->framePlot->SetClickResolution(m_ClickResolution);
 	ui->framePlot->EnableCurText(m_UseCursorText);
 	m_FreqChanged = false;
 
@@ -204,11 +228,12 @@ MainWindow::MainWindow(QWidget *parent) :
 	m_pSdrInterface->SetSpectrumInversion(m_InvertSpectrum);
 	m_pSdrInterface->SetUSFmVersion(m_USFm);
 
-	InitDemodSettings();
 	ui->framePlot->SetDemodCenterFreq( m_DemodFrequency );
 	SetupDemod(m_DemodMode);
 	m_RdsDecode.DecodeReset(m_USFm);
 
+	m_pSdrInterface->SetDemod(m_DemodMode, m_DemodSettings[m_DemodMode]);
+
 	SetupNoiseProc();
 
 	UpdateInfoBox();
@@ -228,6 +253,12 @@ MainWindow::MainWindow(QWidget *parent) :
 		g_pTestBench->show();
 		g_pTestBench->Init();
 	}
+
+	if( (DEMOD_PSK == m_DemodMode) || (DEMOD_FSK == m_DemodMode) )
+		SetChatDialogState(true);
+
+	StopRecord();
+
 }
 
 MainWindow::~MainWindow()
@@ -260,18 +291,16 @@ void MainWindow::closeEvent(QCloseEvent *)
 /////////////////////////////////////////////////////////////////////
 void MainWindow::AlwaysOnTop()
 {
+#ifndef Q_OS_LINUX
 	m_AlwaysOnTop = ui->actionAlwaysOnTop->isChecked();
 	Qt::WindowFlags flags = this->windowFlags();
+
 	if (m_AlwaysOnTop)
-	{
-		this->setWindowFlags(flags | Qt::CustomizeWindowHint | Qt::WindowStaysOnTopHint);
-		this->show();
-	}
+		this->setWindowFlags( (flags & ~Qt::WindowStaysOnBottomHint) | Qt::WindowStaysOnTopHint  | Qt::CustomizeWindowHint );
 	else
-	{
-		this->setWindowFlags( (flags | Qt::CustomizeWindowHint) & ~Qt::WindowStaysOnTopHint );
-		this->show();
-	}
+		this->setWindowFlags( (flags & ~Qt::WindowStaysOnTopHint) | Qt::WindowStaysOnBottomHint  | Qt::CustomizeWindowHint );
+	this->show();
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -291,6 +320,11 @@ void MainWindow::writeSettings()
 		settings.setValue(tr("TestBenchRect"),m_TestBenchRect);
 	}
 
+	if( g_pChatDialog->isVisible() )
+		m_ChatDialogRect = g_pChatDialog->geometry();
+	settings.setValue(tr("ChatDialogRect"),m_ChatDialogRect);
+
+
 	settings.endGroup();
 
 	settings.beginGroup(tr("Common"));
@@ -310,15 +344,20 @@ void MainWindow::writeSettings()
 	settings.setValue(tr("FftSize"),m_FftSize);
 	settings.setValue(tr("FftAve"),m_FftAve);
 	settings.setValue(tr("MaxDisplayRate"),m_MaxDisplayRate);
-	settings.setValue(tr("ClickResolution"),m_ClickResolution);
 	settings.setValue(tr("UseTestBench"),m_UseTestBench);
 	settings.setValue(tr("AlwaysOnTop"),m_AlwaysOnTop);
 	settings.setValue(tr("Volume"),m_Volume);
 	settings.setValue(tr("Percent2DScreen"),m_Percent2DScreen);
+	settings.setValue(tr("ActiveHostAdrIndex"),m_ActiveHostAdrIndex);
 	settings.setValue(tr("InvertSpectrum"),m_InvertSpectrum);
 	settings.setValue(tr("USFm"),m_USFm);
 	settings.setValue(tr("UseCursorText"),m_UseCursorText);
 
+	settings.setValue("RecordFilePath", m_RecordFilePath);
+	settings.setValue("UseUdpFwd", m_UseUdpFwd);
+	settings.setValue(tr("IPFwdAdr"),m_IPFwdAdr.toIPv4Address());
+	settings.setValue(tr("FwdPort"),m_FwdPort);
+
 
 	//Get NCO spur offsets and save
 	m_pSdrInterface->ManageNCOSpurOffsets(CSdrInterface::NCOSPUR_CMD_READ,
@@ -329,6 +368,8 @@ void MainWindow::writeSettings()
 
 	settings.setValue(tr("DemodFrequency"),(int)m_DemodFrequency);
 	settings.setValue(tr("DemodMode"),m_DemodMode);
+	settings.setValue(tr("RecordMode"),m_RecordMode);
+
 
 	settings.setValue(tr("NBOn"),m_NoiseProcSettings.NBOn);
 	settings.setValue(tr("NBThreshold"),m_NoiseProcSettings.NBThreshold);
@@ -354,7 +395,7 @@ void MainWindow::writeSettings()
 	settings.setValue(tr("PulsePeriod"),g_pTestBench->m_PulsePeriod);
 	settings.setValue(tr("SignalPower"),g_pTestBench->m_SignalPower);
 	settings.setValue(tr("NoisePower"),g_pTestBench->m_NoisePower);
-	settings.setValue(tr("UseFmGen"),g_pTestBench->m_UseFmGen);
+	settings.setValue(tr("GenMode"),g_pTestBench->m_GenMode);
 
 	settings.endGroup();
 
@@ -365,6 +406,7 @@ void MainWindow::writeSettings()
 		settings.setArrayIndex(i);
 		settings.setValue(tr("HiCut"), m_DemodSettings[i].HiCut);
 		settings.setValue(tr("LowCut"), m_DemodSettings[i].LowCut);
+		settings.setValue(tr("FreqClickResolution"), m_DemodSettings[i].FreqClickResolution);
 		settings.setValue(tr("FilterClickResolution"), m_DemodSettings[i].FilterClickResolution);
 		settings.setValue(tr("Offset"), m_DemodSettings[i].Offset);
 		settings.setValue(tr("SquelchValue"), m_DemodSettings[i].SquelchValue);
@@ -387,6 +429,12 @@ void MainWindow::readSettings()
 	bool ismin = settings.value(tr("minstate"), false).toBool();
 	m_TestBenchRect = settings.value(tr("TestBenchRect"), QRect(0,0,500,200)).toRect();
 
+	m_ChatDialogRect = settings.value(tr("ChatDialogRect"), QRect(10,10,500,200)).toRect();
+	if( (m_ChatDialogRect.x()<0) ||  (m_ChatDialogRect.y()<0) )
+	{
+		 m_ChatDialogRect.setX(10);
+		 m_ChatDialogRect.setY(10);
+	}
 	settings.endGroup();
 
 	settings.beginGroup(tr("Common"));
@@ -395,6 +443,8 @@ void MainWindow::readSettings()
 	m_SpanFrequency = settings.value(tr("SpanFrequency"), 100000).toUInt();
 	m_IPAdr.setAddress(settings.value(tr("IPAdr"), 0xC0A80164).toInt() );
 	m_Port = settings.value(tr("Port"), 50000).toUInt();
+	m_IPFwdAdr.setAddress(settings.value(tr("IPFwdAdr"), 0xC0A80164).toInt() );
+	m_FwdPort = settings.value(tr("FwdPort"), 50010).toUInt();
 	m_RfGain = settings.value(tr("RfGain"), 0).toInt();
 	m_BandwidthIndex = settings.value(tr("BandwidthIndex"), 0).toInt();
 	m_SoundInIndex = settings.value(tr("SoundInIndex"), 0).toInt();
@@ -406,9 +456,9 @@ void MainWindow::readSettings()
 	m_FftSize = settings.value(tr("FftSize"), 4096).toInt();
 	m_MaxDisplayRate = settings.value(tr("MaxDisplayRate"), 10).toInt();
 	m_RadioType = settings.value(tr("RadioType"), 0).toInt();
-	m_ClickResolution = settings.value(tr("ClickResolution"),100).toInt();
 	m_Volume = settings.value(tr("Volume"),100).toInt();
 	m_Percent2DScreen = settings.value(tr("Percent2DScreen"),50).toInt();
+	m_ActiveHostAdrIndex = settings.value(tr("ActiveHostAdrIndex"),0).toInt();
 
 	m_NCOSpurOffsetI = settings.value(tr("NCOSpurOffsetI"),0.0).toDouble();
 	m_NCOSpurOffsetQ = settings.value(tr("NCOSpurOffsetQ"),0.0).toDouble();
@@ -420,12 +470,16 @@ void MainWindow::readSettings()
 	m_USFm = settings.value(tr("USFm"), true).toBool();
 	m_UseCursorText = settings.value(tr("UseCursorText"), false).toBool();
 
+	m_UseUdpFwd = settings.value(tr("UseUdpFwd"), false).toBool();
+
 	m_NoiseProcSettings.NBOn = settings.value(tr("NBOn"), false).toBool();
 	m_NoiseProcSettings.NBThreshold = settings.value(tr("NBThreshold"),0).toInt();
 	m_NoiseProcSettings.NBWidth = settings.value(tr("NBWidth"),50).toInt();
 
 	m_DemodMode = settings.value(tr("DemodMode"), DEMOD_AM).toInt();
+	m_RecordMode = settings.value(tr("RecordMode"), 0).toInt();
 	m_DemodFrequency = (qint64)settings.value(tr("DemodFrequency"), 15000000).toUInt();
+	m_RecordFilePath = settings.value("RecordFilePath",QCoreApplication::applicationDirPath()+"/Record.wav").toString();
 
 	settings.endGroup();
 
@@ -447,7 +501,7 @@ void MainWindow::readSettings()
 	g_pTestBench->m_PulsePeriod = settings.value(tr("PulsePeriod"),0.0).toDouble();
 	g_pTestBench->m_SignalPower = settings.value(tr("SignalPower"),0.0).toDouble();
 	g_pTestBench->m_NoisePower = settings.value(tr("NoisePower"),0.0).toDouble();
-	g_pTestBench->m_UseFmGen = settings.value(tr("UseFmGen"),false).toBool();
+	g_pTestBench->m_GenMode = settings.value(tr("GenMode"),0).toInt();
 
 	settings.endGroup();
 
@@ -458,9 +512,9 @@ void MainWindow::readSettings()
 		settings.setArrayIndex(i);
 		m_DemodSettings[i].HiCut = settings.value(tr("HiCut"), 5000).toInt();
 		m_DemodSettings[i].LowCut = settings.value(tr("LowCut"), -5000).toInt();
-		m_DemodSettings[i].FilterClickResolution = settings.value(tr("FilterClickResolution"), 100).toInt();
+		m_DemodSettings[i].FreqClickResolution = settings.value(tr("FreqClickResolution"), m_DemodSettings[i].DefFreqClickResolution).toInt();
 		m_DemodSettings[i].Offset = settings.value(tr("Offset"), 0).toInt();
-		m_DemodSettings[i].SquelchValue = settings.value(tr("SquelchValue"), 0).toInt();
+		m_DemodSettings[i].SquelchValue = settings.value(tr("SquelchValue"), -160).toInt();
 		m_DemodSettings[i].AgcSlope = settings.value(tr("AgcSlope"), 0).toInt();
 		m_DemodSettings[i].AgcThresh = settings.value(tr("AgcThresh"), -100).toInt();
 		m_DemodSettings[i].AgcManualGain = settings.value(tr("AgcManualGain"), 30).toInt();
@@ -488,7 +542,7 @@ void MainWindow::OnTimer()
 		if( CSdrInterface::NOT_CONNECTED == m_Status )
 			m_pSdrInterface->ConnectToServer(m_IPAdr,m_Port);
 	}
-	ui->frameMeter->SetdBmLevel( m_pSdrInterface->GetSMeterAve() );
+	ui->frameMeter->SetdBmLevel( m_pSdrInterface->GetSMeterAve(), false );
 	if(DEMOD_WFM == m_DemodMode)	//if in WFM mode manage stereo status display
 	{
 		bool update = false;
@@ -555,6 +609,9 @@ CAboutDlg dlg(this,PROGRAM_TITLE_VERSION);
 /////////////////////////////////////////////////////////////////////
 void MainWindow::OnExit()
 {
+	writeSettings();
+	if(m_pSdrInterface)
+		m_pSdrInterface->StopIO();
 	qApp->exit(0);
 }
 
@@ -567,7 +624,7 @@ void MainWindow::OnDisplayDlg()
 CDisplayDlg dlg(this);
 	dlg.m_FftSize = m_FftSize;
 	dlg.m_FftAve = m_FftAve;
-	dlg.m_ClickResolution = m_ClickResolution;
+	dlg.m_ClickResolution = m_DemodSettings[m_DemodMode].FreqClickResolution;
 	dlg.m_MaxDisplayRate = m_MaxDisplayRate;
 	dlg.m_UseTestBench = m_UseTestBench;
 	dlg.m_Percent2DScreen = m_Percent2DScreen;
@@ -591,13 +648,13 @@ CDisplayDlg dlg(this);
 		m_FftSize = dlg.m_FftSize;
 		m_FftAve = dlg.m_FftAve;
 		m_UseCursorText = dlg.m_UseCursorText;
-		m_ClickResolution = dlg.m_ClickResolution;
+		m_DemodSettings[m_DemodMode].FreqClickResolution = dlg.m_ClickResolution;
 		m_MaxDisplayRate = dlg.m_MaxDisplayRate;
 		m_UseTestBench = dlg.m_UseTestBench;
 		m_pSdrInterface->SetFftAve( m_FftAve);
 		m_pSdrInterface->SetFftSize( m_FftSize);
 		m_pSdrInterface->SetMaxDisplayRate(m_MaxDisplayRate);
-		ui->framePlot->SetClickResolution(m_ClickResolution);
+		ui->framePlot->SetClickResolution(m_DemodSettings[m_DemodMode].FreqClickResolution);
 		ui->framePlot->EnableCurText(m_UseCursorText);
 		if(m_UseTestBench)
 		{	//make TestBench visable if not already
@@ -653,6 +710,7 @@ void MainWindow::OnSdrDlg()
 CSdrSetupDlg dlg(this,m_pSdrInterface);
 	dlg.m_BandwidthIndex = m_BandwidthIndex;
 	dlg.m_USFm = m_USFm;
+
 	dlg.InitDlg();
 	dlg.SetSpectrumInversion(m_InvertSpectrum);
 	if( dlg.exec() )
@@ -700,7 +758,11 @@ void MainWindow::OnNetworkDlg()
 CEditNetDlg dlg(this);
 	dlg.m_IPAdr = m_IPAdr;
 	dlg.m_Port = m_Port;
+	dlg.m_IPFwdAdr = m_IPFwdAdr;
+	dlg.m_FwdPort = m_FwdPort;
 	dlg.m_ActiveDevice = m_ActiveDevice;
+	dlg.m_UseUdpFwd = m_UseUdpFwd;
+	dlg.m_ActiveHostAdrIndex = m_ActiveHostAdrIndex;
 	dlg.InitDlg();
 	if( dlg.exec() )
 	{
@@ -714,7 +776,11 @@ CEditNetDlg dlg(this);
 			m_pSdrInterface->StopIO();
 			m_IPAdr = dlg.m_IPAdr;
 			m_Port = dlg.m_Port;
+			m_IPFwdAdr = dlg.m_IPFwdAdr;
+			m_FwdPort = dlg.m_FwdPort;
+			m_UseUdpFwd = dlg.m_UseUdpFwd;
 			m_ActiveDevice = dlg.m_ActiveDevice;
+			m_ActiveHostAdrIndex = dlg.m_ActiveHostAdrIndex;
 		}
 	}
 }
@@ -741,6 +807,59 @@ CNoiseProcDlg dlg(this);
 	dlg.exec();
 }
 
+/////////////////////////////////////////////////////////////////////
+// Called by Record Setup Menu event
+/////////////////////////////////////////////////////////////////////
+void MainWindow::OnRecordSetupDlg()
+{
+CRecordSetupDlg dlg(this);
+	dlg.m_RecordMode = m_RecordMode;
+	dlg.m_RecordFilePath = m_RecordFilePath;
+	dlg.Init();
+	if( dlg.exec() )
+	{
+		m_RecordFilePath = dlg.m_RecordFilePath;
+		if(m_RecordMode != dlg.m_RecordMode)
+		{
+			if(CSdrInterface::RUNNING == m_Status)
+			{
+				m_pSdrInterface->StopSdr();
+				ui->framePlot->SetRunningState(false);
+			}
+			m_RecordMode = dlg.m_RecordMode;
+		}
+	}
+}
+
+/////////////////////////////////////////////////////////////////////
+// Called by Record Button event
+/////////////////////////////////////////////////////////////////////
+void MainWindow::OnRecord()
+{
+	if(m_Recording)
+	{
+		StopRecord();
+	}
+	else
+	{
+		if(m_pSdrInterface->StartFileRecord(m_RecordFilePath, m_RecordMode, m_CenterFrequency) )
+		{
+			m_Recording = true;
+			ui->pushButtonRecord->setText("Stop Record");
+			ui->pushButtonRecord->setStyleSheet("background-color: rgb(255, 0, 0);");
+		}
+	}
+}
+/////////////////////////////////////////////////////////////////////
+// Called to stop file recording
+/////////////////////////////////////////////////////////////////////
+void MainWindow::StopRecord()
+{
+	m_pSdrInterface->StopFileRecord();
+	m_Recording = false;
+	ui->pushButtonRecord->setText("Start Record");
+	ui->pushButtonRecord->setStyleSheet("background-color: rgb(180, 180, 180);");
+}
 
 /////////////////////////////////////////////////////////////////////
 // Called to update the information text box
@@ -764,6 +883,7 @@ void MainWindow::OnRun()
 	{
 		m_CenterFrequency = m_pSdrInterface->SetRxFreq(m_CenterFrequency);
 		m_pSdrInterface->SetDemodFreq(m_CenterFrequency - m_DemodFrequency);
+		m_pSdrInterface->SetForwardingParameters( m_UseUdpFwd, m_IPFwdAdr, m_FwdPort);
 		m_pSdrInterface->StartSdr();
 		m_pSdrInterface->m_MissedPackets = 0;
 
@@ -773,6 +893,7 @@ void MainWindow::OnRun()
 	}
 	else if(CSdrInterface::RUNNING == m_Status)
 	{
+		StopRecord();
 		m_pSdrInterface->StopSdr();
 		ui->framePlot->SetRunningState(false);
 		ReadPerformance();
@@ -1032,11 +1153,44 @@ void MainWindow::SetupNoiseProc()
 	m_pSdrInterface->SetupNoiseProc(&m_NoiseProcSettings);
 }
 
+/////////////////////////////////////////////////////////////////////
+// Called when memory dialog checkbox is clicked
+/////////////////////////////////////////////////////////////////////
+void MainWindow::SetChatDialogState(int state)
+{
+	if(state)
+	{	//make memory dialog visable if not already
+		if(!g_pChatDialog->isVisible())
+		{
+			g_pChatDialog->setGeometry(m_ChatDialogRect);
+			g_pChatDialog->show();
+		}
+		g_pChatDialog->activateWindow();
+		g_pChatDialog->raise();
+	}
+	else
+	{	//hide memory dialog
+		if(g_pChatDialog->isVisible())
+		{
+			m_ChatDialogRect = g_pChatDialog->geometry();
+			g_pChatDialog->hide();
+		}
+	}
+}
+
 /////////////////////////////////////////////////////////////////////
 // Setup Demod parameters and clamp to limits
 /////////////////////////////////////////////////////////////////////
 void MainWindow::SetupDemod(int index)
 {
+qDebug()<<index;
+	if(m_DemodMode != index)
+	{
+		if( (DEMOD_PSK == index) || (DEMOD_FSK == index) )
+			SetChatDialogState(true);
+		else
+			SetChatDialogState(false);
+	}
 	m_DemodMode = index;
 	ui->framePlot->SetDemodRanges(
 			m_DemodSettings[m_DemodMode].LowCutmin,
@@ -1062,6 +1216,8 @@ void MainWindow::SetupDemod(int index)
 	m_pSdrInterface->SetDemod(m_DemodMode, m_DemodSettings[m_DemodMode]);
 	m_pSdrInterface->SetDemodFreq(m_CenterFrequency - m_DemodFrequency);
 	UpdateInfoBox();
+	ui->frameMeter->SetSquelchPos( m_DemodSettings[m_DemodMode].SquelchValue );
+	ui->framePlot->SetClickResolution(m_DemodSettings[m_DemodMode].FreqClickResolution);
 }
 
 
@@ -1078,6 +1234,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_AM].LowCutmax = -500;
 	m_DemodSettings[DEMOD_AM].LowCutmin = -10000;
 	m_DemodSettings[DEMOD_AM].Symetric = true;
+	m_DemodSettings[DEMOD_AM].DefFreqClickResolution = 1000;
+	m_DemodSettings[DEMOD_AM].FilterClickResolution = 100;
 
 	m_DemodSettings[DEMOD_SAM].txt = tr("AM");
 	m_DemodSettings[DEMOD_SAM].HiCutmin = 100;
@@ -1085,6 +1243,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_SAM].LowCutmax = -100;
 	m_DemodSettings[DEMOD_SAM].LowCutmin = -10000;
 	m_DemodSettings[DEMOD_SAM].Symetric = false;
+	m_DemodSettings[DEMOD_SAM].DefFreqClickResolution = 1000;
+	m_DemodSettings[DEMOD_SAM].FilterClickResolution = 100;
 
 	m_DemodSettings[DEMOD_FM].txt = tr("FM");
 	m_DemodSettings[DEMOD_FM].HiCutmin = 5000;
@@ -1092,6 +1252,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_FM].LowCutmax = -5000;
 	m_DemodSettings[DEMOD_FM].LowCutmin = -15000;
 	m_DemodSettings[DEMOD_FM].Symetric = true;
+	m_DemodSettings[DEMOD_FM].DefFreqClickResolution = 5000;
+	m_DemodSettings[DEMOD_FM].FilterClickResolution = 5000;
 
 	m_DemodSettings[DEMOD_WFM].txt = tr("WFM");
 	m_DemodSettings[DEMOD_WFM].HiCutmin = 100000;
@@ -1099,6 +1261,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_WFM].LowCutmax = -100000;
 	m_DemodSettings[DEMOD_WFM].LowCutmin = -100000;
 	m_DemodSettings[DEMOD_WFM].Symetric = true;
+	m_DemodSettings[DEMOD_WFM].DefFreqClickResolution = 100000;
+	m_DemodSettings[DEMOD_WFM].FilterClickResolution = 10000;
 
 	m_DemodSettings[DEMOD_USB].txt = tr("USB");
 	m_DemodSettings[DEMOD_USB].HiCutmin = 500;
@@ -1106,6 +1270,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_USB].LowCutmax = 200;
 	m_DemodSettings[DEMOD_USB].LowCutmin = 0;
 	m_DemodSettings[DEMOD_USB].Symetric = false;
+	m_DemodSettings[DEMOD_USB].DefFreqClickResolution = 100;
+	m_DemodSettings[DEMOD_USB].FilterClickResolution = 100;
 
 	m_DemodSettings[DEMOD_LSB].txt = tr("LSB");
 	m_DemodSettings[DEMOD_LSB].HiCutmin = -200;
@@ -1113,6 +1279,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_LSB].LowCutmax = -500;
 	m_DemodSettings[DEMOD_LSB].LowCutmin = -20000;
 	m_DemodSettings[DEMOD_LSB].Symetric = false;
+	m_DemodSettings[DEMOD_LSB].DefFreqClickResolution = 100;
+	m_DemodSettings[DEMOD_LSB].FilterClickResolution = 100;
 
 	m_DemodSettings[DEMOD_CWU].txt = tr("CWU");
 	m_DemodSettings[DEMOD_CWU].HiCutmin = 50;
@@ -1120,6 +1288,8 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_CWU].LowCutmax = -50;
 	m_DemodSettings[DEMOD_CWU].LowCutmin = -1000;
 	m_DemodSettings[DEMOD_CWU].Symetric = false;
+	m_DemodSettings[DEMOD_CWU].DefFreqClickResolution = 10;
+	m_DemodSettings[DEMOD_CWU].FilterClickResolution = 50;
 
 	m_DemodSettings[DEMOD_CWL].txt = tr("CWL");
 	m_DemodSettings[DEMOD_CWL].HiCutmin = 50;
@@ -1127,6 +1297,24 @@ void MainWindow::InitDemodSettings()
 	m_DemodSettings[DEMOD_CWL].LowCutmax = -50;
 	m_DemodSettings[DEMOD_CWL].LowCutmin = -1000;
 	m_DemodSettings[DEMOD_CWL].Symetric = false;
+	m_DemodSettings[DEMOD_CWL].DefFreqClickResolution = 10;
+	m_DemodSettings[DEMOD_CWL].FilterClickResolution = 50;
 
-	m_pSdrInterface->SetDemod(m_DemodMode, m_DemodSettings[m_DemodMode]);
+	m_DemodSettings[DEMOD_PSK].txt = tr("PSK");
+	m_DemodSettings[DEMOD_PSK].HiCutmin = 50;
+	m_DemodSettings[DEMOD_PSK].HiCutmax = 50;
+	m_DemodSettings[DEMOD_PSK].LowCutmax = -50;
+	m_DemodSettings[DEMOD_PSK].LowCutmin = -50;
+	m_DemodSettings[DEMOD_PSK].Symetric = true;
+	m_DemodSettings[DEMOD_PSK].DefFreqClickResolution = 1;
+	m_DemodSettings[DEMOD_PSK].FilterClickResolution = 5;
+
+	m_DemodSettings[DEMOD_FSK].txt = tr("Raw DSC");
+	m_DemodSettings[DEMOD_FSK].HiCutmin = 20;
+	m_DemodSettings[DEMOD_FSK].HiCutmax = 200;
+	m_DemodSettings[DEMOD_FSK].LowCutmax = -20;
+	m_DemodSettings[DEMOD_FSK].LowCutmin = -200;
+	m_DemodSettings[DEMOD_FSK].Symetric = true;
+	m_DemodSettings[DEMOD_FSK].DefFreqClickResolution = 10;
+	m_DemodSettings[DEMOD_FSK].FilterClickResolution = 10;
 }
