@@ -49,7 +49,7 @@
 //////////////////////////////////////////////////////////////////////
 #define MAX_SCREENSIZE 4096
 #define CUR_CUT_DELTA 10		//cursor capture delta in pixels
-#define OVERLOAD_DISPLAY_LIMIT 10
+#define OVERLOAD_DISPLAY_LIMIT 3
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -156,7 +156,7 @@ QPoint gpt = event->globalPos();
 				if(CENTER!=m_CursorCaptured)
 					setCursor(QCursor(Qt::CrossCursor));
 				m_CursorCaptured = CENTER;
-				DisplayCursorFreq(gpt, FreqfromX( pt.x() ));
+				DisplayCursorFreq(gpt, RoundFreq( FreqfromX( pt.x() ), m_ClickResolution) );
 			}
 			else if( IsPointCloseTo( pt.x(),m_DemodHiCutFreqX, m_CursorCaptureDelta) )
 			{	//in move demod hicut region
@@ -306,6 +306,7 @@ QPoint gpt = event->globalPos();
 		if( CENTER!=m_CursorCaptured)
 		{	//if cursor not captured set demod frequency and start demod box capture
 			m_DemodCenterFreq = RoundFreq(FreqfromX(pt.x()),m_ClickResolution );
+//qDebug()<<m_ClickResolution << m_DemodCenterFreq;
 			emit NewDemodFreq(m_DemodCenterFreq);
 			//save initial grab postion from m_DemodFreqX
 			setCursor(QCursor(Qt::CrossCursor));
@@ -445,7 +446,8 @@ void CPlotter::paintEvent(QPaintEvent *)
 	painter.drawPixmap(0,0,m_2DPixmap);
 	painter.drawPixmap(0, m_Percent2DScreen*m_Size.height()/100,m_WaterfallPixmap);
 	//tell interface that its ok to signal a new line of fft data
-	m_pSdrInterface->ScreenUpdateDone();
+	if(m_pSdrInterface)
+		m_pSdrInterface->ScreenUpdateDone();
 	return;
 }
 
@@ -474,7 +476,9 @@ QPoint LineBuf[MAX_SCREENSIZE];
 
 	QPainter painter1(&m_WaterfallPixmap);
 	//get scaled FFT data
-	bool fftoverload = m_pSdrInterface->GetScreenIntegerFFTData( 255, w,
+	bool fftoverload = false;
+	if(m_pSdrInterface)
+		fftoverload = m_pSdrInterface->GetScreenIntegerFFTData( 255, w,
 							m_MaxdB,
 							m_MindB,
 							-m_Span/2,
@@ -495,8 +499,15 @@ QPoint LineBuf[MAX_SCREENSIZE];
 	m_2DPixmap = m_OverlayPixmap.copy(0,0,w,h);
 
 	QPainter painter2(&m_2DPixmap);
+	// workaround for "fixed" line drawing since Qt 5
+	// see http://stackoverflow.com/questions/16990326
+#if QT_VERSION >= 0x050000
+	painter2.translate(0.5, 0.5);
+#endif
+
 	//get new scaled fft data
-	m_pSdrInterface->GetScreenIntegerFFTData( h, w,
+	if(m_pSdrInterface)
+		m_pSdrInterface->GetScreenIntegerFFTData( h, w,
 							m_MaxdB,
 							m_MindB,
 							-m_Span/2,
@@ -564,8 +575,9 @@ QRect rect;
 	painter.setBrush(Qt::SolidPattern);
 	painter.setOpacity(0.5);
 
-	int LockState;
-	m_pSdrInterface->GetStereoLock(&LockState);
+    int LockState = false;
+	if(m_pSdrInterface)
+		m_pSdrInterface->GetStereoLock(&LockState);
 	if( LockState )
 		painter.fillRect(m_DemodLowCutFreqX, 0,dw, h, Qt::green);
 	else
@@ -632,7 +644,7 @@ QRect rect;
 	{
 		if( 0==i )
 		{
-			if((hskip==1) )
+            if( hskip==1 )
 			{	//left justify the leftmost text
 				x = (int)( (float)i*pixperdiv);
 				rect.setRect(x ,y, (int)pixperdiv, h/VERT_DIVS);
@@ -641,7 +653,7 @@ QRect rect;
 		}
 		else if(HORZ_DIVS == i)
 		{
-			if((hskip==1) )
+            if( hskip==1 )
 			{	//right justify the rightmost text
 				x = (int)( (float)i*pixperdiv - pixperdiv);
 				rect.setRect(x ,y, (int)pixperdiv, h/VERT_DIVS);
@@ -678,7 +690,7 @@ QRect rect;
 	for( int i=0; i<VERT_DIVS-1; i+=vskip)
 	{
 		y = (int)( (float)i*pixperdiv );
-		painter.drawStaticText(5, y-1, QString::number(dB)+" dB");
+		painter.drawText(5, y-1, QString::number(dB)+" dB");
 		dB -= (m_dBStepSize*vskip);
 	}
 	m_MindB = m_MaxdB - (VERT_DIVS)*m_dBStepSize;
