@@ -14,6 +14,8 @@
 //	2011-08-07  Added WFM Support
 //	2012-06-01  fixed threading issue with m_TxMsg
 //	2013-04-13  Added CloudSDR support, fixed network closing bug with pending msgs
+//	2015-03-26  Added  support for small MTU and UDP keepalive in case of port forwarding timeouts
+//	2015-10-26  Added Files saving functionality
 /////////////////////////////////////////////////////////////////////
 
 //==========================================================================================
@@ -49,11 +51,12 @@
 #include <QDebug>
 
 #define SPUR_CAL_MAXSAMPLES 300000
-#define MAX_SAMPLERATES 4
+#define MAX_SAMPLERATES 5
 
 //Tables to get various parameters based on the gui sdrsetup samplerate index value
 const quint32 SDRIQ_MAXBW[MAX_SAMPLERATES] =
 {
+	25000,
 	50000,
 	100000,
 	150000,
@@ -62,6 +65,7 @@ const quint32 SDRIQ_MAXBW[MAX_SAMPLERATES] =
 
 const quint32 SDRIQ_6620FILTERS[MAX_SAMPLERATES] =
 {
+	Cad6620::BWKHZ_25,
 	Cad6620::BWKHZ_50,
 	Cad6620::BWKHZ_100,
 	Cad6620::BWKHZ_150,
@@ -70,6 +74,7 @@ const quint32 SDRIQ_6620FILTERS[MAX_SAMPLERATES] =
 
 const TYPEREAL SDRIQ_6620FILTERGAIN[MAX_SAMPLERATES] =
 {
+	0.0,
 	0.0,
 	8.0,
 	11.0,
@@ -78,6 +83,7 @@ const TYPEREAL SDRIQ_6620FILTERGAIN[MAX_SAMPLERATES] =
 
 const TYPEREAL SDRIQ_SAMPLERATE[MAX_SAMPLERATES] =
 {
+	(66666666.6667/2400.0),
 	(66666666.6667/1200.0),
 	(66666666.6667/600.0),
 	(66666666.6667/420.0),
@@ -87,7 +93,7 @@ const TYPEREAL SDRIQ_SAMPLERATE[MAX_SAMPLERATES] =
 const quint32 NETSDR_MAXBW[MAX_SAMPLERATES] =
 {
 	50000,
-//	100000,
+	100000,
 	200000,
 	500000,
 	1600000
@@ -96,6 +102,7 @@ const quint32 NETSDR_MAXBW[MAX_SAMPLERATES] =
 const TYPEREAL NETSDR_SAMPLERATE[MAX_SAMPLERATES] =
 {
 	(80.0e6/1280.0),
+	(80.0e6/640.0),
 	(80.0e6/320.0),
 	(80.0e6/128.0),
 	(80.0e6/40.0)
@@ -104,6 +111,7 @@ const TYPEREAL NETSDR_SAMPLERATE[MAX_SAMPLERATES] =
 const quint32 SDRIP_MAXBW[MAX_SAMPLERATES] =
 {
 	50000,
+	100000,
 	200000,
 	500000,
 	1800000
@@ -112,6 +120,7 @@ const quint32 SDRIP_MAXBW[MAX_SAMPLERATES] =
 const TYPEREAL SDRIP_SAMPLERATE[MAX_SAMPLERATES] =
 {
 	(80.0e6/1280.0),
+	(80.0e6/640.0),
 	(80.0e6/320.0),
 	(80.0e6/130.0),
 	(80.0e6/40.0)
@@ -120,17 +129,19 @@ const TYPEREAL SDRIP_SAMPLERATE[MAX_SAMPLERATES] =
 const quint32 CLOUDSDR_MAXBW[MAX_SAMPLERATES] =
 {
 	40000,
+	100000,
 	200000,
 	500000,
-	1000000
+	1500000
 };
 
 const TYPEREAL CLOUDSDR_SAMPLERATE[MAX_SAMPLERATES] =
 {
 	(122.88e6/2560.0),
+	(122.88e6/984.0),
 	(122.88e6/500.0),
 	(122.88e6/200.0),
-	(122.88e6/100.0)
+	(122.88e6/68.0)
 };
 
 
@@ -140,6 +151,9 @@ const TYPEREAL CLOUDSDR_SAMPLERATE[MAX_SAMPLERATES] =
 CSdrInterface::CSdrInterface()
 {
 	m_Running = false;
+	m_FileRecordActive = false;
+	m_RecordMode = RECORDMODE_AUDIO;
+	m_24BitData = false;
 	m_BootRev = 0.0;
 	m_AppRev = 0.0;
 	m_FftBufPos = 0;
@@ -168,6 +182,7 @@ CSdrInterface::CSdrInterface()
 	SetFftAve(1);
 	m_pSoundCardOut = new CSoundOut();
 	m_pdataProcess = new CDataProcess(this);
+	m_pWaveFileWriter = new CWaveFileWriter;
 	m_Status = NOT_CONNECTED;
 	m_ChannelMode = CI_RX_CHAN_SETUP_SINGLE_1;	//default channel settings for NetSDR
 	m_Channel = CI_RX_CHAN_1;
@@ -176,6 +191,14 @@ CSdrInterface::CSdrInterface()
 CSdrInterface::~CSdrInterface()
 {
 qDebug()<<"CSdrInterface destructor";
+	if(m_pWaveFileWriter)
+	{
+		if(m_FileRecordActive)
+			StopFileRecord();
+		if(m_pWaveFileWriter->isOpen())
+			m_pWaveFileWriter->Close();
+		delete m_pWaveFileWriter;
+	}
 	if(m_pSoundCardOut)
 	{
 		delete m_pSoundCardOut;
@@ -186,6 +209,45 @@ qDebug()<<"CSdrInterface destructor";
 		delete m_pdataProcess;
 		m_pdataProcess = NULL;
 	}
+
+}
+
+////////////////////////////////////////////////////////////////////////
+// Start/Stop wav file recording
+////////////////////////////////////////////////////////////////////////
+bool CSdrInterface::StartFileRecord(QString Filename, int RecordMode, qint64 CenterFreq)
+{
+bool ret = false;
+	m_RecordMode = RecordMode;
+	if(	m_Running )
+	{
+		if(RECORDMODE_IQ == m_RecordMode)
+		{
+			if(m_pWaveFileWriter->Open(Filename, true, m_SampleRate, m_24BitData, CenterFreq) )
+			{
+				m_FileRecordActive = true;
+				ret = true;
+qDebug()<<"Start IQ Record";
+			}
+		}
+		else
+		{
+			if(m_pWaveFileWriter->Open(Filename, m_StereoOut, 48000, false, CenterFreq) )
+			{
+				m_FileRecordActive = true;
+				ret = true;
+qDebug()<<"Start Audio Record";
+			}
+		}
+	}
+	return ret;
+}
+
+void CSdrInterface::StopFileRecord()
+{
+qDebug()<<"Stop Record";
+	m_pWaveFileWriter->Close();
+	m_FileRecordActive = false;
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -282,7 +344,7 @@ CAscpTxMsg TxMsg;
 					m_RadioType = SDRIP;
 				else if("NetSDR" == m_DeviceName)
 					m_RadioType = NETSDR;
-				if("CloudSDR" == m_DeviceName)
+				if( ("CloudSDR" == m_DeviceName) || ("CloudIQ" == m_DeviceName) )
 					m_RadioType = CLOUDSDR;
 				if( (SDRIP==m_RadioType) || (NETSDR==m_RadioType) || (CLOUDSDR==m_RadioType))
 				{
@@ -298,6 +360,8 @@ CAscpTxMsg TxMsg;
 			case CI_GENERAL_INTERFACE_VERSION:
 				break;
 			case CI_GENERAL_HARDFIRM_VERSION:
+//qDebug()<<"=" <<pMsg->GetParm16()<<pMsg->GetParm8();
+#if 1
 				if(pMsg->GetParm8() == 0)
 				{
 					m_BootRev = (float)pMsg->GetParm16()/100.0;
@@ -308,6 +372,7 @@ CAscpTxMsg TxMsg;
 					m_BandwidthIndex = -1;	//force update of sample rate logic
 					emit NewInfoData();
 				}
+#endif
 				break;
 			case CI_RX_STATE:
 				pMsg->GetParm8();
@@ -325,7 +390,6 @@ CAscpTxMsg TxMsg;
 			case CI_GENERAL_OPTIONS:
 				break;
 			case CI_GENERAL_SECURITY_CODE:
-//qDebug()<<"security = "<<pMsg->GetParm16();
 				break;
 			case CI_GENERAL_STATUS_CODE:	//used as keepalive ack
 				m_KeepAliveCounter = 0;
@@ -485,12 +549,12 @@ CAscpTxMsg TxMsg;
 
     TxMsg.InitTxMsg(TYPE_HOST_REQ_CITEM);
     TxMsg.AddCItem(CI_GENERAL_HARDFIRM_VERSION);
-    TxMsg.AddParm8(0);
+	TxMsg.AddParm8(0);
     SendAscpMsg(&TxMsg);
 
     TxMsg.InitTxMsg(TYPE_HOST_REQ_CITEM);
     TxMsg.AddCItem(CI_GENERAL_HARDFIRM_VERSION);
-    TxMsg.AddParm8(1);
+	TxMsg.AddParm8(1);
     SendAscpMsg(&TxMsg);
 
 	m_BaseFrequencyRangeMin = 0;		//load default frequency ranges
@@ -593,17 +657,33 @@ CAscpTxMsg TxMsg;
             TxMsg.AddParm32((quint32)m_SampleRate);
             SendAscpMsg(&TxMsg);
 
+			TxMsg.InitTxMsg(TYPE_HOST_SET_CITEM);
+			TxMsg.AddCItem(CI_RX_OUTPUT_PARAMS);
+			if(m_SampleRate>100000)
+				TxMsg.AddParm8(CI_RX_OUTPUT_PARAMS_LARGE_MTU);
+			else
+				TxMsg.AddParm8(CI_RX_OUTPUT_PARAMS_SMALL_MTU);
+			SendAscpMsg(&TxMsg);
+
             TxMsg.InitTxMsg(TYPE_HOST_SET_CITEM);
             TxMsg.AddCItem(CI_RX_STATE);
             TxMsg.AddParm8(RX_STATE_DATACOMPLEX);
             TxMsg.AddParm8(RX_STATE_ON);
 			if(m_SampleRate<1500000.0)
+			{
                 TxMsg.AddParm8(MODE_CONTIGUOUS24);
+				m_24BitData = true;
+			}
 			else
+			{
                 TxMsg.AddParm8(MODE_CONTIGUOUS16);
-            TxMsg.AddParm8(0);
+				m_24BitData = false;
+			}
+			TxMsg.AddParm8(0);
             SendAscpMsg(&TxMsg);
 
+			emit SendUdpKeepalive();
+
 			m_NCOSpurOffsetI = 0.0;		//don't need for netsdr
 			m_NCOSpurOffsetQ = 0.0;
 			m_NcoSpurCalActive = false;
@@ -618,6 +698,7 @@ CAscpTxMsg TxMsg;
             TxMsg.AddParm32(24);
             SendAscpMsg(&TxMsg);
 
+			m_24BitData = false;
             TxMsg.InitTxMsg(TYPE_HOST_SET_CITEM);
             TxMsg.AddCItem(CI_RX_STATE);
             TxMsg.AddParm8(RX_STATE_COMPLEX_HF);
@@ -642,8 +723,10 @@ CAscpTxMsg TxMsg;
 void CSdrInterface::StopSdr()
 {
 CAscpTxMsg TxMsg;
-    m_Running = false;
-	m_pSoundCardOut->Stop();
+	if(m_FileRecordActive)
+		StopFileRecord();
+	m_Running = false;
+    m_pSoundCardOut->Stop();
     TxMsg.InitTxMsg(TYPE_HOST_SET_CITEM);
     TxMsg.AddCItem(CI_RX_STATE);
     TxMsg.AddParm8(RX_STATE_DATACOMPLEX);
@@ -651,6 +734,11 @@ CAscpTxMsg TxMsg;
     TxMsg.AddParm8(0);
     TxMsg.AddParm8(0);
     SendAscpMsg(&TxMsg);
+
+	TxMsg.InitTxMsg(TYPE_HOST_SET_CITEM);
+	TxMsg.AddCItem(CI_RX_OUTPUT_PARAMS);
+	TxMsg.AddParm8(CI_RX_OUTPUT_PARAMS_LARGE_MTU);
+	SendAscpMsg(&TxMsg);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -689,6 +777,8 @@ CAscpTxMsg TxMsg;
 						false,
 						m_GainCalibrationOffset-m_RfGain,
 						m_SampleRate);
+	m_Demodulator.SetSmeterOffset(m_GainCalibrationOffset-m_RfGain);
+
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -731,6 +821,7 @@ CAscpTxMsg TxMsg;
 	return freq;
 }
 
+
 ///////////////////////////////////////////////////////////////////////////////
 // Send Keep alive message to SDR
 ///////////////////////////////////////////////////////////////////////////////
@@ -740,7 +831,7 @@ CAscpTxMsg TxMsg;
     TxMsg.InitTxMsg(TYPE_HOST_REQ_CITEM);
     TxMsg.AddCItem(CI_GENERAL_STATUS_CODE);
 	SendAscpMsg(&TxMsg);
-	if(++m_KeepAliveCounter >2)		//see if no ack received
+    if(++m_KeepAliveCounter >4)		//see if no ack received
 	{
 		SendStatus(ERR);
 qDebug()<<"Keepalive failed";
@@ -816,6 +907,11 @@ void CSdrInterface::SetDemod(int Mode, tDemodInfo CurrentDemodInfo)
 qDebug()<<"UsrDataRate="<< m_Demodulator.GetOutputRate();
 }
 
+void CSdrInterface::SetPskMode(int Index)
+{
+	m_Demodulator.SetPskMode(Index);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 //called to change Noise Processing parameters
 ///////////////////////////////////////////////////////////////////////////////
@@ -941,6 +1037,7 @@ void CSdrInterface::ProcessIQData(TYPECPX *pIQData, int NumSamples)
 	}
 
 	g_pTestBench->CreateGeneratorSamples(NumSamples, (TYPECPX*)pIQData, m_SampleRate);
+
 	m_NoiseProc.ProcessBlanker(NumSamples, (TYPECPX*)pIQData, (TYPECPX*)pIQData);
 
 	if(m_NcoSpurCalActive)	//if performing NCO spur calibration
@@ -948,8 +1045,9 @@ void CSdrInterface::ProcessIQData(TYPECPX *pIQData, int NumSamples)
 	//accumulate samples into m_DataBuf until have enough to perform an FFT
 	for(int i=0; i<NumSamples; i++)
 	{
-		m_DataBuf[m_FftBufPos].re = pIQData[i].re - m_NCOSpurOffsetI;
-		m_DataBuf[m_FftBufPos++].im = pIQData[i].im - m_NCOSpurOffsetQ;
+		pIQData[i].re = pIQData[i].re - m_NCOSpurOffsetI;
+		pIQData[i].im = pIQData[i].im - m_NCOSpurOffsetQ;
+		m_DataBuf[m_FftBufPos++] = pIQData[i];
 		if(m_FftBufPos >= (m_FftSize) )
 		{
 			m_FftBufPos = 0;
@@ -971,12 +1069,36 @@ void CSdrInterface::ProcessIQData(TYPECPX *pIQData, int NumSamples)
 	{
 		n = m_Demodulator.ProcessData(NumSamples, pIQData, SoundBuf);
 		if(m_pSoundCardOut)
-			m_pSoundCardOut->PutOutQueue(n, SoundBuf);
+		{
+			n = m_pSoundCardOut->PutOutQueue(n, SoundBuf);
+			if( m_FileRecordActive && (RECORDMODE_AUDIO == m_RecordMode) )
+			{	//write stereo audio to file if active
+				if(!m_pWaveFileWriter->Write(n*4, (qint8*)m_pSoundCardOut->m_RData ))
+					StopFileRecord();
+			}
+		}
 	}
 	else
 	{
 		n = m_Demodulator.ProcessData(NumSamples, pIQData, (TYPEREAL*)SoundBuf);
 		if(m_pSoundCardOut)
-			m_pSoundCardOut->PutOutQueue(n, (TYPEREAL*)SoundBuf);
+		{
+			n = m_pSoundCardOut->PutOutQueue(n, (TYPEREAL*)SoundBuf);
+			if( m_FileRecordActive && (RECORDMODE_AUDIO == m_RecordMode) )
+			{	//write mono audio to file if active
+				if(!m_pWaveFileWriter->Write(n*2, (qint8*)m_pSoundCardOut->m_RData ))
+					StopFileRecord();
+			}
+		}
+	}
+}
+
+void CSdrInterface::ProcessUdpData(char* pBuf, qint64 Length)
+{
+	if(m_pdataProcess) m_pdataProcess->PutInQ(pBuf,Length);
+	if( (m_FileRecordActive) && (RECORDMODE_IQ == m_RecordMode) )
+	{	//write IQ data to file if active
+		if(!m_pWaveFileWriter->Write(Length-4, (qint8*)(pBuf+4) ))	//write packet minus header
+			StopFileRecord();
 	}
 }
